


/*
 * "Zap.c"
 */

#include <ctype.h>          /* Fuer `isspace ()'. */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include "Zap.h"

/* ----------------------------------------------------- */

/*
 * Hole das naechste Wort, auf das "*input" zeigt. Die
 * aufrufende Funktion ist dafuer verantwortlich, den
 * allozierten Speicher freizugeben. "*input" zeigt
 * nach dem Aufruf genau hinter das gerade gelesene Wort.
 * Ist kein Wort mehr da, wird "NULL" zurueckgegeben.
 */
static String GetString (String *input);

/*
 * Erzeuge ein neues Widget, aehnlich "XtCreateWidget ()".
 * Allerdings wird dieser Funktion der *Name* der
 * Widgetklasse uebergeben und nicht der
 * Klassenpointer.
 */
static Widget CreateWidget
(
  String name,
  String className,
  Widget parent
);

/* ----------------------------------------------------- */

void Zap (Widget widget)
{
  typedef struct
  {
    Boolean manage;
    String  children;
  } ZapResourceRecord;
  ZapResourceRecord zapResourceRecord;
  static XtResource zapResources[] =
  {
    {
      "manage", "Manage",
      XtRBoolean, sizeof (Boolean),
      XtOffsetOf (ZapResourceRecord, manage),
      XtRImmediate, (XtPointer) False
    },
    {
      "children", "Children",
      XtRString, sizeof (String),
      XtOffsetOf (ZapResourceRecord, children),
      XtRString, (XtPointer) NULL
    }
  };

  /*
   * Hole Subressourcen "zap.manage" und "zap.children".
   */
  XtVaGetSubresources
  (
    widget, (XtPointer) &zapResourceRecord,
    "zap", "Zap", zapResources, XtNumber (zapResources),
    NULL
  );

  /*
   * Manage das Widget, wenn verlangt.
   */
  if (zapResourceRecord.manage) XtManageChild (widget);

  /*
   * Erzeuge die Child-Widgets, wie in der Subressource
   * "zap.children" angegeben.
   */
  if (zapResourceRecord.children != NULL)
  {
    String p = zapResourceRecord.children;

    for (;;)
    {
      String widgetClassName;
      String widgetName;
      Widget child;

      /*
       * Lies Child-Widget-Beschreibung:
       *
       * { <widget-class-name> <widget-name> }
       */
      widgetClassName = GetString (&p);
      if (widgetClassName == NULL) break;

      widgetName = GetString (&p);
      if (widgetName == NULL)
      {
        XtFree (widgetClassName);
        break; 
      }

      /*
       * Erzeuge das Child-Widget.
       */
      child = CreateWidget
      (
        widgetName,
        widgetClassName,
        widget
      );
      XtFree (widgetName);
      XtFree (widgetClassName);
      if (child == NULL) continue;

      /*
       * Zappe das gerade erzeugte Child-Widget.
       */
      Zap (child);
    }
  }
}

/* ----------------------------------------------------- */

static String GetString (String *input)
{
  String   p = *input;
  Cardinal length;
  String   string;

  while (isspace (*p)) p++;
  if (*p == '\0') return (NULL);

  for
  (
    length = 0;
    p[length] && !isspace (p[length]);
    length++
  );
  string = (String) XtMalloc (length + 1);
  memcpy (string, p, length);
  string[length] = '\0';

  *input = p + length;
  return (string);
}

/* ----------------------------------------------------- */

#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>

/*
 * Man beachte, dass nur *diese* Funktion die Motif-Header-
 * dateien benoetigt; die Applikation und die "Zap"-
 * Funktion sind Widget-Set-unabhaengig!
 */

static Widget CreateWidget
(
  String name,
  String className,
  Widget parent
)
{
  Widget widget;

  /*
   * Die Zuordnung "Name->Erzeugungsfunktion" hat
   * gegenueber der Zuordnung "Name->Widgetklasse" den
   * Vorteil, dass auch "Convenience functions" wie
   * "XmCreateFormDialog ()" verfuegbar sind, die gleich
   * mehrere Widgets auf einmal erzeugen.
   */
  static const struct
  {
    String className;
    Widget (*creationFunction)
                  (Widget, String, Arg *, Cardinal);
  } classNameToCreationFunction[] =
  {
    "XmFrame",      XmCreateFrame,
    "XmLabel",      XmCreateLabel,
    "XmPushButton", XmCreatePushButton,
    "XmRowColumn",  XmCreateRowColumn
  };
  int i;

  /*
   * Finde die zum Klassennamen passende Widget-Erzeugungs-
   * Funktion.
   */
  for (i = 0; i < XtNumber (classNameToCreationFunction); i++)
  {
    if (!strcmp
    (
      className,
      classNameToCreationFunction[i].className
    )) break;
  }
  if (i == XtNumber (classNameToCreationFunction))
  {
    XtAppWarning
    (
      XtWidgetToApplicationContext (parent),
      "Unknown class"
    );
    return (NULL);
  }

  /*
   * Erzeuge das Widget.
   */
  widget = classNameToCreationFunction[i].
     creationFunction (parent, name, NULL, 0);

  return (widget);
}

/* ----------------------------------------------------- */

